<?php

declare(strict_types=1);

namespace Drupal\google_v3_glossary\Form;

use Drupal\Core\File\FileSystem;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\google_v3_glossary\Services\GoogleV3GlossaryFileManagerService;
use Drupal\google_v3_glossary\Traits\GoogleV3TranslatorCSVTrait;

/**
 * Provides a google_v3_glossary form.
 */
class LanguageGlossaryForm extends FormBase {
  use GoogleV3TranslatorCSVTrait;

  /**
   * Service for handling the file authentication, upload and download.
   *
   * @var \Drupal\google_v3_glossary\Services\GoogleV3GlossaryFileManagerService
   */
  protected readonly GoogleV3GlossaryFileManagerService $fileManager;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystem|mixed
   */
  protected FileSystem $fileSystem;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'google_v3_glossary_glossary';
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(FileSystem $fileSystem, GoogleV3GlossaryFileManagerService $fileManager) {
    $this->fileSystem = $fileSystem;
    $this->fileManager = $fileManager;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ],
    ];

    return $form;
  }

  /**
   * Load data from CSV file.
   *
   * @param string $fileName
   *   The name of the file to load.
   * @param string $langCode
   *   The language code for the file.
   * @param string $destinationPath
   *   The path to the destination where the file will be saved.
   */
  protected function loadData(string $fileName, string $langCode, string $destinationPath): string {
    try {
      // Google V3 Glossary.
      $this->fileManager->downloadGlossaryFromGoogle($destinationPath, $langCode);
    }
    catch (\Exception $e) {
      $this->logger('google_v3_glossary')->error($e->getMessage());
    }

    $data = $this->loadCsvFile($fileName);

    if ($data) {
      return $this->convertArrayToCSVString($data);
    }

    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Validate that the textarea is valid CSV.
    $validated = $this->validateCsv($form_state->getValue('translations'));

    if (!$validated['valid']) {
      $form_state->setErrorByName('translations', $validated['message']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Empty.
  }

}
