<?php

declare(strict_types=1);

namespace Drupal\google_v3_glossary\Services;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\google_v3_glossary\Traits\GoogleStorageTrait;
use Google\Cloud\Storage\StorageClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\google_v3_glossary\Traits\GoogleV3GlossaryTrait;

/**
 * Manage auth, and file operations for the Google V3 Glossary files.
 */
class GoogleV3GlossaryFileManagerService implements ContainerInjectionInterface {
  use GoogleStorageTrait;
  use GoogleV3GlossaryTrait;

  /**
   * Drupal file system.
   *
   * @var \Drupal\Core\File\FileSystem|mixed
   */
  private FileSystemInterface $fileSystem;

  /**
   * Google cloud storage.
   *
   * @var \Google\Cloud\Storage\StorageClient
   */
  private StorageClient $storageClient;

  /**
   * Google V3 setting project id.
   *
   * @var string
   */
  private string $projectId;

  /**
   * Google V3 setting location.
   *
   * @var string
   */
  private string $location;

  /**
   * Google storage bucket name.
   *
   * @var string
   */
  private string $bucketName;

  /**
   * Google V3 module settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * Constructs a GoogleV3GlossaryFileManagerService object.
   */
  final public function __construct(FileSystemInterface $fileSystem, ConfigFactoryInterface $configFactory) {
    // Initialize Google Storage trait.
    $this->initGoogleStorage();
    $this->config = $configFactory->get('google_v3_glossary.settings');

    $this->fileSystem = $fileSystem;
    $this->projectId = $this->getProjectId();
    $this->location = $this->getLocation();
    $this->bucketName = $this->config->get('bucket_name');

    // Key file path from tmgmt_google_v3 module.
    $path = $this->getGoogleKeyFile();
    if (is_null($path)) {
      throw new \Exception('Google key file not found. Check the tmgmt_google_v3 module settings.');
    }

    $absolutePath = $this->fileSystem->realpath($path);

    $this->storageClient = new StorageClient(
      [
        'keyFilePath' => $absolutePath,
      ]
    );
    $this->storageClient->registerStreamWrapper();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('config.factory'),
    );
  }

  /**
   * Download a glossary file from Cloud Storage.
   *
   * @param string $destinationPath
   *   The path where the file will be downloaded.
   * @param string $langCode
   *   The two-letter language code for the glossary.
   *
   * @throws \Exception
   *   If the download fails.
   */
  public function downloadGlossaryFromGoogle($destinationPath, $langCode): bool {
    try {
      $bucket = $this->storageClient->bucket($this->bucketName);
      $object = $bucket->object($this->getStorageFilePath($langCode));
      $object->downloadToFile($destinationPath);

      return TRUE;
    }
    catch (\Exception $e) {
      throw new \Exception('Failed to download glossary file: ' . $e->getMessage());
    }
  }

  /**
   * Upload a glossary file to Cloud Storage.
   *
   * @param string $destinationPath
   *   The path of the file to upload.
   * @param string $langCode
   *   The two-letter language code for the glossary.
   *
   * @throws \Exception
   *   If the upload fails.
   */
  public function uploadGlossaryToGoogle($destinationPath, $langCode): bool {
    try {
      $bucket = $this->storageClient->bucket($this->bucketName);
      $storageObject = $bucket->upload(
        fopen($destinationPath, 'r'),
        [
          'name' => $this->getStorageFilePath($langCode),
        ]
      );

      if ($storageObject) {
        return TRUE;
      }

      return FALSE;
    }
    catch (\Exception $e) {
      throw new \Exception('Failed to upload glossary file: ' . $e->getMessage());
    }
  }

  /**
   * Google cloud storage file path.
   *
   * @param string $languageCode
   *   Two letter language code.
   *
   * @return string
   *   Cloud storage file path relative to the bucket.
   */
  private function getStorageFilePath(string $languageCode): string {
    $enabledLanguages = $this->getEnabledLanguages();

    foreach ($enabledLanguages as $langCode => $langName) {
      $paths[$langCode] = $this->config->get('file_path_' . $langCode);
    }

    return $paths[$languageCode];
  }

}
