<?php

declare(strict_types=1);

namespace Drupal\google_v3_glossary\Traits;

use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\file\FileStorage;
use Drupal\tmgmt\Entity\Translator;

/**
 * Trait GoogleV3Translator.
 */
trait GoogleStorageTrait {
  use LoggerChannelTrait;

  /**
   * Tmgmt module translator entity.
   *
   * @var \Drupal\tmgmt\Entity\Translator|null
   */
  private ?Translator $translator;

  /**
   * Initialize settings like a constructor.
   *
   * @return void
   *   Returns nothing.
   */
  public function initGoogleStorage(): void {
    try {
      $this->translator = \Drupal::entityTypeManager()->getStorage('tmgmt_translator')->load('google_v3');
    }
    catch (\Exception $e) {
      $this->getLogger('google_v3_glossary')->error($e->getMessage());

      $this->translator = NULL;
    }
  }

  /**
   * Get the key file uploaded in the tmgmt_google_v3 module config.
   *
   * @return string|null
   *   The file path.
   */
  public function getGoogleKeyFile(): ?string {
    if (is_null($this->translator)) {
      return NULL;
    }

    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $credentialsFid = $this->translator->getSetting('google_credentials');

    if (empty($credentialsFid)) {
      return NULL;
    }

    $fileStorage = $this->getFileEntity();
    $file = $fileStorage->load($credentialsFid[0]);

    return $file->getFileUri();
  }

  /**
   * Get the location setting from tmgmt_google_v3 module config.
   *
   * @return string
   *   Location setting.
   */
  public function getLocation(): string {
    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $location = $this->translator->getSetting('location');

    return $location ?? '';
  }

  /**
   * Get the Project Id from tmgmt_google_v3 module config.
   *
   * @return string
   *   Project id setting.
   */
  public function getProjectId(): string {
    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $id = $this->translator->getSetting('api_project');

    return $id ?? '';
  }

  /**
   * File entity getter.
   *
   * @return \Drupal\file\FileStorage
   *   File storage.
   */
  protected function getFileEntity(): FileStorage {
    /** @var \Drupal\file\FileStorage $file */
    return \Drupal::service('entity_type.manager')->getStorage('file');
  }

}
