<?php

declare(strict_types=1);

namespace Drupal\google_v3_glossary\Traits;

use Drupal\Core\File\FileSystemInterface;

/**
 * Trait GoogleV3Translator.
 */
trait GoogleV3TranslatorCSVTrait {

  /**
   * Load a CSV file from local file system.
   *
   * @param string $fileName
   *   Name of the file to load.
   *
   * @return array|false
   *   Array from CSV file.
   */
  public function loadCsvFile(string $fileName): array|bool {
    // Check if the file exists.
    if (file_exists($fileName)) {
      // Optionally, parse the CSV data (example using fgetcsv).
      $rows = [];
      $handle = fopen($fileName, 'r');
      if ($handle) {
        while (($data = fgetcsv($handle)) !== FALSE) {
          $rows[] = $data;
        }
        fclose($handle);
      }

      return $rows;
    }

    return FALSE;
  }

  /**
   * Save data to CSV file.
   *
   * @param string $data
   *   Array to save to CSV file.
   * @param string $filePath
   *   File path for saving the CSV to Drupal's files.
   */
  public function saveCsvFile(string $data, string $filePath) {
    // Get the private file system path.
    $fileSystemService = $this->getFileSystemService();
    $newfilePath = $fileSystemService->realpath($filePath);

    $dataArray = $this->convertStringToArray($data);

    // Write array to file as CSV.
    $handle = fopen($newfilePath, 'w');
    foreach ($dataArray as $row) {
      fputcsv($handle, $row);
    }
    fclose($handle);
  }

  /**
   * Convert CSV string to array.
   *
   * @param string $data
   *   CSV string to convert to array.
   *
   * @return array
   *   Array from CSV string.
   */
  public function convertStringToArray(string $data): array {
    $rows = explode("\r\n", $data);
    $array = [];
    foreach ($rows as $row) {
      if (empty($row)) {
        continue;
      }
      $array[] = explode(',', $row);
    }

    // Clean up the array.
    foreach ($array as &$row) {
      // Remove extra spaces.
      $row = array_map('trim', $row);
    }

    return $array;
  }

  /**
   * Convert array to CSV string.
   *
   * Textarea field values are strings and need to convert bewtween strings
   * and arrays.
   *
   * @param array $array
   *   Array to convert to CSV string.
   *
   * @return string
   *   CSV string from an array.
   */
  public function convertArrayToCsvString(array $array): string {
    $csvString = '';
    foreach ($array as $row) {
      $csvString .= implode(',', $row) . "\n";
    }

    return $csvString;
  }

  /**
   * Validates a CSV string from a form textarea.
   *
   * @param string $csvString
   *   The CSV string to validate.
   * @param string $delimiter
   *   The delimiter character (default: ',').
   * @param bool $hasHeader
   *   Whether the first row is a header (default: true).
   *
   * @return array
   *   Validation result with validity status and messages.
   */
  public function validateCsv($csvString, $delimiter = ',', $hasHeader = FALSE): array {
    // Check if string is empty.
    // An empty textarea value is OK since it is not required.
    if (empty(trim($csvString))) {
      return ['valid' => TRUE, 'message' => 'CSV string is empty'];
    }

    // Split into rows, normalize line endings.
    $rows = explode("\n", str_replace(["\r\n", "\r"], "\n", $csvString));

    // Remove empty rows at the end.
    while (count($rows) > 0 && trim(end($rows)) === '') {
      array_pop($rows);
    }

    if (count($rows) === 0) {
      return ['valid' => FALSE, 'message' => 'CSV contains no data rows'];
    }

    // Parse each row.
    $parsedRows = [];
    $columnCount = NULL;
    $headers = [];

    foreach ($rows as $index => $row) {
      // Skip empty rows.
      if (trim($row) === '') {
        continue;
      }

      // Parse with str_getcsv to handle quotes correctly.
      $parsedRow = str_getcsv($row, $delimiter);

      // First row as header.
      if ($index === 0 && $hasHeader) {
        $headers = $parsedRow;
        $columnCount = count($headers);
        continue;
      }

      // Set column count from first data row if no headers.
      if ($columnCount === NULL) {
        $columnCount = count($parsedRow);
      }

      // Check for consistent column count.
      if (count($parsedRow) !== $columnCount) {
        return [
          'valid' => FALSE,
          'message' => 'Inconsistent column count at row ' .
          ($hasHeader ? $index : $index + 1) .
          '. Expected ' . $columnCount . ', found ' . count($parsedRow),
        ];
      }

      $parsedRows[] = $parsedRow;
    }

    // Check if we only have a header row.
    if (count($parsedRows) === 0 && $hasHeader) {
      return ['valid' => FALSE, 'message' => 'CSV contains only header row'];
    }

    // Success result.
    $result = [
      'valid' => TRUE,
      'message' => 'Valid CSV with ' . count($parsedRows) . ' row(s), ' . $columnCount . ' column(s)',
      'data' => $parsedRows,
    ];

    if ($hasHeader) {
      $result['headers'] = $headers;
    }

    return $result;
  }

  /**
   * Get the file system service as a getter.
   *
   * @return \Drupal\Core\File\FileSystemInterface
   *   File system.
   */
  protected function getFileSystemService(): FileSystemInterface {
    /** @var \Drupal\Core\File\FileSystemInterface $file_system */
    return \Drupal::service('file_system');
  }

  /**
   * Check if file exists and create it if not.
   *
   * @param string $path
   *   The path to the file relative to the private files.
   *
   * @return void
   *   No return value (void).
   */
  public function validateCreateDestinationPath(string $path): void {
    $pathParts = pathinfo($path);
    $privatePath = $this->fileSystem->realpath('private://');
    $destinationPath = $privatePath . '/' . $pathParts['dirname'];
    $destinationFile = $privatePath . '/' . $path;
    // Create directory if it doesn't exist.
    if (!is_dir($privatePath . '/' . $pathParts['dirname'])) {
      $this->fileSystem->prepareDirectory($destinationPath, FileSystemInterface:: CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
    }

    // Create the file if it doesn't exist.
    if (!file_exists($destinationFile)) {
      file_put_contents($destinationFile, '');
    }
    // Check that the file exists.
    if (!file_exists($destinationFile)) {
      // Write error message to the log.
      $this->logger('google_v3_glossary')->error('File %file does not exist.', ['%file' => $destinationFile]);
    }
  }

}
